==> dbt-fal-1.3.4.tar.gz: dbt-fal-1.3.4/src/dbt/adapters/fal_experimental/adapter.py <==
from __future__ import annotations

from contextlib import contextmanager
from functools import partial
from typing import Any

from dbt.adapters.base.impl import BaseAdapter
from dbt.config.runtime import RuntimeConfig
from dbt.contracts.connection import AdapterResponse

from isolate.backends.virtualenv import PythonIPC, VirtualPythonEnvironment
from dbt.adapters.fal_experimental.utils.environments import get_default_pip_dependencies
from dbt.parser.manifest import MacroManifest, Manifest

from isolate.backends import BaseEnvironment

from .adapter_support import (
    prepare_for_adapter,
    read_relation_as_df,
    reconstruct_adapter,
    write_df_to_relation,
)

from .utils import retrieve_symbol

FAL_SCRIPTS_PATH_VAR_NAME = 'fal-scripts-path'

def run_with_adapter(code: str, adapter: BaseAdapter, config: RuntimeConfig) -> Any:
    # main symbol is defined during dbt-fal's compilation
    # and acts as an entrypoint for us to run the model.
    fal_scripts_path = str(_get_fal_scripts_path(config))
    with extra_path(fal_scripts_path):
        main = retrieve_symbol(code, "main")
        return main(
            read_df=prepare_for_adapter(adapter, read_relation_as_df),
            write_df=prepare_for_adapter(adapter, write_df_to_relation),
        )

def _isolated_runner(code: str, config: RuntimeConfig, manifest: Manifest, macro_manifest: MacroManifest) -> Any:
    # This function can be run in an entirely separate
    # process or an environment, so we need to reconstruct
    # the DB adapter solely from the config.
    adapter = reconstruct_adapter(config, manifest, macro_manifest)
    return run_with_adapter(code, adapter, config)

def run_in_environment_with_adapter(
    environment: BaseEnvironment,
    code: str,
    config: RuntimeConfig,
    manifest: Manifest,
    macro_manifest: MacroManifest
) -> AdapterResponse:
    from isolate.backends.remote import IsolateServer
    """Run the 'main' function inside the given code on the
    specified environment.

    The environment_name must be defined inside fal_project.yml file
    in your project's root directory."""
    if type(environment) == IsolateServer:
        # TODO: make a specialized function for this case?
        deps = [i for i in get_default_pip_dependencies() if i.startswith('dbt-')]

        if environment.target_environment_kind == 'conda':
            raise NotImplementedError("Remote environment with `conda` is not supported yet.")
        else:
            environment.target_environment_config['requirements'].extend(deps)

        key = environment.create()

        with environment.open_connection(key) as connection:
            execute_model = partial(_isolated_runner, code, config, manifest, macro_manifest)
            result = connection.run(execute_model)
            return result


    else:
        deps = get_default_pip_dependencies()
        stage = VirtualPythonEnvironment(deps)
        fal_scripts_path = _get_fal_scripts_path(config)

        with PythonIPC(environment, environment.create(), extra_inheritance_paths=[fal_scripts_path, stage.create()]) as connection:
            execute_model = partial(_isolated_runner, code, config, manifest, macro_manifest)
            result = connection.run(execute_model)
            return result

@contextmanager
def extra_path(path: str):
    import sys
    sys.path.append(path)
    try:
        yield
    finally:
        sys.path.remove(path)

def _get_fal_scripts_path(config: RuntimeConfig):
    import pathlib
    project_path = pathlib.Path(config.project_root)

    # Default value
    fal_scripts_path = 'fal_scripts'

    if hasattr(config, 'vars'):
        fal_scripts_path: str = config.vars.to_dict().get(FAL_SCRIPTS_PATH_VAR_NAME, fal_scripts_path)  # type: ignore

    if hasattr(config, 'cli_vars'):
        fal_scripts_path = config.cli_vars.get(FAL_SCRIPTS_PATH_VAR_NAME, fal_scripts_path)

    return project_path / fal_scripts_path

==> dbt-fal-1.3.8.tar.gz: dbt-fal-1.3.8/src/dbt/adapters/fal_experimental/adapter.py <==
from __future__ import annotations

from functools import partial
from typing import Any

from dbt.adapters.base.impl import BaseAdapter
from dbt.config.runtime import RuntimeConfig
from dbt.contracts.connection import AdapterResponse

from isolate.backends.virtualenv import PythonIPC, VirtualPythonEnvironment
from dbt.adapters.fal_experimental.utils.environments import get_default_pip_dependencies
from dbt.parser.manifest import MacroManifest, Manifest

from isolate.backends import BaseEnvironment

from .adapter_support import (
    prepare_for_adapter,
    read_relation_as_df,
    reconstruct_adapter,
    write_df_to_relation,
)

from .utils import extra_path, get_fal_scripts_path, retrieve_symbol

def run_with_adapter(code: str, adapter: BaseAdapter, config: RuntimeConfig) -> Any:
    # main symbol is defined during dbt-fal's compilation
    # and acts as an entrypoint for us to run the model.
    fal_scripts_path = str(get_fal_scripts_path(config))
    with extra_path(fal_scripts_path):
        main = retrieve_symbol(code, "main")
        return main(
            read_df=prepare_for_adapter(adapter, read_relation_as_df),
            write_df=prepare_for_adapter(adapter, write_df_to_relation),
        )

def _isolated_runner(code: str, config: RuntimeConfig, manifest: Manifest, macro_manifest: MacroManifest) -> Any:
    # This function can be run in an entirely separate
    # process or an environment, so we need to reconstruct
    # the DB adapter solely from the config.
    adapter = reconstruct_adapter(config, manifest, macro_manifest)
    return run_with_adapter(code, adapter, config)

def run_in_environment_with_adapter(
    environment: BaseEnvironment,
    code: str,
    config: RuntimeConfig,
    manifest: Manifest,
    macro_manifest: MacroManifest
) -> AdapterResponse:
    from isolate.backends.remote import IsolateServer
    """Run the 'main' function inside the given code on the
    specified environment.

    The environment_name must be defined inside fal_project.yml file
    in your project's root directory."""
    if type(environment) == IsolateServer:
        # TODO: make a specialized function for this case?
        deps = [i for i in get_default_pip_dependencies() if i.startswith('dbt-')]

        extra_config = {
            'kind': 'virtualenv',
            'configuration': { 'requirements': deps }
        }

        environment.target_environments.append(extra_config)

        key = environment.create()

        with environment.open_connection(key) as connection:
            execute_model = partial(_isolated_runner, code, config, manifest, macro_manifest)
            result = connection.run(execute_model)
            return result


    else:
        deps = get_default_pip_dependencies()
        stage = VirtualPythonEnvironment(deps)
        fal_scripts_path = get_fal_scripts_path(config)

        with PythonIPC(environment, environment.create(), extra_inheritance_paths=[fal_scripts_path, stage.create()]) as connection:
            execute_model = partial(_isolated_runner, code, config, manifest, macro_manifest)
            result = connection.run(execute_model)
            return result
